#include "Fit/Fitter.h"
#include "Fit/BinData.h"
#include "Fit/Chi2FCN.h"
#include "TH1.h"
#include "TList.h"
#include "Math/WrappedMultiTF1.h"
#include "HFitInterface.h"
#include "TCanvas.h"
#include "TStyle.h"
 
 
// definition of shared parameter
// background function
int nParB = 13;
int iparB[13] = { 16, // Xi(1820) top        amplitude
                 4, // Xi(1820) top+bottom mean        Shared
                 5,  // Xi(1820) top+bottom sigma       Shared
                 17, // Xi(1690) top          amplitude
                  7, // Xi(1690) top+bottom mean       Shared
                  8, // Xi(1690) top+bottom sigma      Shared
                 18, // Xi(1620) top          amplitude
                  10, // Xi(1620) top+bottom mean       Shared
                  11, // Xi(1620) top+bottom sigma      Shared
                 19,
                 20,
                 21,
                 22
};
 
// signal + background function
int nParSB = 16;
int iparSB[16] = { 0, // Xi(1530) amplitude
                  1, // Xi(1530) mean
                  2, // Xi(1530) sigma
                  3, // Xi(1820)     bottom amplitude
                  4, // Xi(1820) top+bottom mean       Shared
                  5, // Xi(1820) top+bottom sigma      Shared
                  6, // Xi(1690)     bottom amplitude
                  7, // Xi(1690) top+bottom mean       Shared
                  8, // Xi(1690) top+bottom sigma      Shared
                  9, // Xi(1620)     bottom amplitude
                  10, // Xi(1620) top+bottom mean       Shared
                  11, // Xi(1620) top+bottom sigma      Shared
                  12,
                  13,
                  14,
                  15
};

const int Npar = 23;

double gAmplitude1 = 40;
double gCenter1 = 1.53;
double gSigma1 = 0.01;

double gAmplitude2 = 40;
double gAmplitude2top = 200;
double gCenter2 = 1.82;
double gSigma2 = 0.015;

double gAmplitude3 = 40;
double gAmplitude3top = 40;
double gCenter3 = 1.7;
double gSigma3 = 0.015;

double gAmplitude4 = 20;
double gAmplitude4top = 70;
double gCenter4 = 1.62;
double gSigma4 = 0.01;

double polySB1 = -771.915; 
double polySB2 = 654.205; 
double polySB3 = 225.88; 
double polySB4 = -163.869; 

double polyB1 = -31238.2; 
double polyB2 = 15504.1; 
double polyB3 = 11206.6; 
double polyB4 = -5440.22; 

  double par0[Npar] = { gAmplitude1,gCenter1,gSigma1, //1530 
			gAmplitude2,gCenter2,gSigma2,//1820 
			gAmplitude3,gCenter3,gSigma3,//1690 
			gAmplitude4,gCenter4,gSigma4,//1620 
			polySB1,polySB2,polySB3,polySB4,
			gAmplitude2top,
			gAmplitude3top,
			gAmplitude4top,
			polyB1,polyB2,polyB3,polyB4
  };

 
// Create the GlobalCHi2 structure
 
struct GlobalChi2 {
  GlobalChi2(  ROOT::Math::IMultiGenFunction & f1,
	       ROOT::Math::IMultiGenFunction & f2) :
    fChi2_1(&f1), fChi2_2(&f2) {}
 
  double operator() (const double *par) const {
    double p1[nParB];
    for (int i = 0; i < nParB; ++i) p1[i] = par[iparB[i] ];
 
    double p2[nParSB];
    for (int i = 0; i < nParSB; ++i) p2[i] = par[iparSB[i] ];
 
    return (*fChi2_1)(p1) + (*fChi2_2)(p2);
  }
 
  const  ROOT::Math::IMultiGenFunction * fChi2_1;
  const  ROOT::Math::IMultiGenFunction * fChi2_2;
};

void combinedFit() {

  gROOT->Reset();//Reset session memory
  gROOT->SetStyle("Plain");//Make style plain
  gStyle->SetOptStat(0);//Remove the statistics box from histogram

  TFile *hFile1 = new TFile("allKmLam.root");
  TH1D *hB; hFile1->GetObject("CasStar_Kin_MMS_4",hB);

  TFile *hFile2 = new TFile("mAllXi.root");
  TH1D *hSB; hFile2->GetObject("hCasStar_MC_Kin_4",hSB);

  hB->Rebin(8);
  hSB->Rebin(8);

  hB->GetXaxis()->SetRangeUser(1.48,1.9);
  hSB->GetXaxis()->SetRangeUser(1.48,1.9);
 
  TF1 * fB = new TF1("fB","gaus + gaus(3) + gaus(6) + pol3(9)",0,100);


 
  // perform now global fit
  TF1 * fSB = new TF1("fSB","gaus + gaus(3) + gaus(6) + gaus(9) + pol3(12)",0,100);//!!!!!
 
  ROOT::Math::WrappedMultiTF1 wfB(*fB,1);
  ROOT::Math::WrappedMultiTF1 wfSB(*fSB,1);
 
  ROOT::Fit::DataOptions opt;
  ROOT::Fit::DataRange rangeB;
  // set the data range
  rangeB.SetRange(1.6,1.9);//!!!!!
  ROOT::Fit::BinData dataB(opt,rangeB);
  ROOT::Fit::FillData(dataB, hB);
 
  ROOT::Fit::DataRange rangeSB;
  rangeSB.SetRange(1.48,1.9);//!!!!!
  ROOT::Fit::BinData dataSB(opt,rangeSB);
  ROOT::Fit::FillData(dataSB, hSB);
 
  ROOT::Fit::Chi2Function chi2_B(dataB, wfB);
  ROOT::Fit::Chi2Function chi2_SB(dataSB, wfSB);
 
  GlobalChi2 globalChi2(chi2_B, chi2_SB);
 
  ROOT::Fit::Fitter fitter;

 
  // create before the parameter settings in order to fix or set range on them
  fitter.Config().SetParamsSettings(Npar,par0);
  // fix 5-th parameter
  //fitter.Config().ParSettings(2).Fix();

  //fitter.Config().ParSettings(0).Fix();
  //fitter.Config().ParSettings(1).Fix();
  //fitter.Config().ParSettings(2).Fix();

  //fitter.Config().ParSettings(13).Fix();
  //fitter.Config().ParSettings(3).Fix();
  //fitter.Config().ParSettings(4).Fix();
  //fitter.Config().ParSettings(5).Fix();

  //fitter.Config().ParSettings(6).Fix();
  //fitter.Config().ParSettings(7).Fix();
  //fitter.Config().ParSettings(8).Fix();

  //fitter.Config().ParSettings(9).Fix();
  //fitter.Config().ParSettings(10).Fix();
  //fitter.Config().ParSettings(11).Fix();

  //fitter.Config().ParSettings(11).Fix();
  //fitter.Config().ParSettings(12).Fix();
  //fitter.Config().ParSettings(13).Fix();
  //fitter.Config().ParSettings(14).Fix();



  // set limits on the third and 4-th parameter
  //fitter.Config().ParSettings(2).SetLimits(-10,-1.E-4);
  fitter.Config().ParSettings(0).SetLimits(0,1000000);
  fitter.Config().ParSettings(0).SetStepSize(5);

  fitter.Config().ParSettings(3).SetLimits(0,1000000);
  fitter.Config().ParSettings(3).SetStepSize(5);

  fitter.Config().ParSettings(6).SetLimits(0,1000000);
  fitter.Config().ParSettings(6).SetStepSize(5);

  fitter.Config().ParSettings(9).SetLimits(0,1000000);
  fitter.Config().ParSettings(9).SetStepSize(5);

  fitter.Config().ParSettings(11).SetLimits(0.005,0.06);
  fitter.Config().ParSettings(9).SetStepSize(0.001);

  fitter.Config().ParSettings(16).SetLimits(0,1000000);
  fitter.Config().ParSettings(16).SetStepSize(5);

  fitter.Config().ParSettings(17).SetLimits(0,1000000);
  fitter.Config().ParSettings(17).SetStepSize(5);

  fitter.Config().ParSettings(18).SetLimits(0,1000000);
  fitter.Config().ParSettings(18).SetStepSize(5);
 
  fitter.Config().MinimizerOptions().SetPrintLevel(0);
  fitter.Config().SetMinimizer("Minuit2","Migrad");
 
  // fit FCN function directly
  // (specify optionally data size and flag to indicate that is a chi2 fit)

  fitter.FitFCN(Npar,globalChi2,0,dataB.Size()+dataSB.Size(),true);
  ROOT::Fit::FitResult result = fitter.Result();
  result.Print(std::cout);

  TF1 * f1620Top = new TF1("f1620Top","gaus",0,100);
  TF1 * f1690Top = new TF1("f1690Top","gaus",0,100);
  TF1 * f1820Top = new TF1("f1820Top","gaus",0,100);
  TF1 * fBackTop = new TF1("fBackTop","pol3",0,100);

  f1820Top->SetParameter(0,fB->GetParameter(0));
  f1820Top->SetParameter(1,fB->GetParameter(1));
  f1820Top->SetParameter(2,fB->GetParameter(2));

  f1690Top->SetParameter(0,fB->GetParameter(3));
  f1690Top->SetParameter(1,fB->GetParameter(4));
  f1690Top->SetParameter(2,fB->GetParameter(5));

  f1620Top->SetParameter(0,fB->GetParameter(6));
  f1620Top->SetParameter(1,fB->GetParameter(7));
  f1620Top->SetParameter(2,fB->GetParameter(8));

  fBackTop->SetParameter(0,fB->GetParameter(9));
  fBackTop->SetParameter(1,fB->GetParameter(10));
  fBackTop->SetParameter(2,fB->GetParameter(11));
  fBackTop->SetParameter(3,fB->GetParameter(12));

  TF1 * f1530Bottom = new TF1("f1530Bottom","gaus",0,100);
  TF1 * f1620Bottom = new TF1("f1620Bottom","gaus",0,100);
  TF1 * f1690Bottom = new TF1("f1690Bottom","gaus",0,100);
  TF1 * f1820Bottom = new TF1("f1820Bottom","gaus",0,100);
  TF1 * fBackBottom = new TF1("fBackBottom","pol3",0,100);
 
  f1530Bottom->SetParameter(0,fSB->GetParameter(0));
  f1530Bottom->SetParameter(1,fSB->GetParameter(1));
  f1530Bottom->SetParameter(2,fSB->GetParameter(2));

  f1820Bottom->SetParameter(0,fSB->GetParameter(3));
  f1820Bottom->SetParameter(1,fSB->GetParameter(4));
  f1820Bottom->SetParameter(2,fSB->GetParameter(5));

  f1690Bottom->SetParameter(0,fSB->GetParameter(6));
  f1690Bottom->SetParameter(1,fSB->GetParameter(7));
  f1690Bottom->SetParameter(2,fSB->GetParameter(8));

  f1620Bottom->SetParameter(0,fSB->GetParameter(9));
  f1620Bottom->SetParameter(1,fSB->GetParameter(10));
  f1620Bottom->SetParameter(2,fSB->GetParameter(11));

  fBackBottom->SetParameter(0,fSB->GetParameter(12));
  fBackBottom->SetParameter(1,fSB->GetParameter(13));
  fBackBottom->SetParameter(2,fSB->GetParameter(14));
  fBackBottom->SetParameter(3,fSB->GetParameter(15));

  f1690Top->SetLineColor(3);
  f1690Top->SetFillStyle(3001);
  f1690Top->SetFillColor(3);

  f1820Top->SetLineColor(2);
  f1820Top->SetFillStyle(3001);
  f1820Top->SetFillColor(2);

  f1690Bottom->SetLineColor(3);
  f1690Bottom->SetFillStyle(3001);
  f1690Bottom->SetFillColor(3);

  f1620Bottom->SetLineColor(7);
  f1620Bottom->SetFillStyle(3001);
  f1620Bottom->SetFillColor(7);

  f1530Bottom->SetLineColor(4);
  f1530Bottom->SetFillStyle(3001);
  f1530Bottom->SetFillColor(4);

  fBackTop->SetLineStyle(10);
  fBackBottom->SetLineStyle(10);

  hB->GetXaxis()->SetTitle("Mass[K #Lambda]/GeV");
  hB->GetYaxis()->SetTitle("Counts");
  hB->GetXaxis()->SetTitleSize(0.05);
  hB->GetYaxis()->SetTitleSize(0.05);

  hSB->GetXaxis()->SetTitle("Mass[#pi^{0} #Xi^{-}]/GeV");
  hSB->GetYaxis()->SetTitle("Counts");
  hSB->GetXaxis()->SetTitleSize(0.05);
  hSB->GetYaxis()->SetTitleSize(0.05);


  TCanvas * c1 = new TCanvas("Simfit","Simultaneous fit of two histograms",
			     10,10,700,700);
  c1->Divide(1,2);
  c1->cd(1);
  gStyle->SetOptFit(1111);
 
  //fB->SetFitResult( result, iparB);
  fB->SetRange(rangeB().first, rangeB().second);
  fB->SetLineColor(1);
  //hB->GetListOfFunctions()->Add(fB);
  hB->Draw("e");
  fB->Draw("same");
  //f1620Top->Draw("same");
  f1690Top->Draw("same");
  f1820Top->Draw("same");
  fBackTop->Draw("same");

  c1->cd(2);
  //fSB->SetFitResult( result, iparSB);
  fSB->SetRange(rangeSB().first, rangeSB().second);
  fSB->SetLineColor(1);
  //hSB->GetListOfFunctions()->Add(fSB);
  hSB->SetMinimum(0);
  hSB->Draw("e");
  fSB->Draw("same");
  f1620Bottom->Draw("same"); 
  f1530Bottom->Draw("same");
  f1690Bottom->Draw("same");
  f1820Bottom->Draw("same");
  fBackBottom->Draw("same");
}

